/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2019/10/15
* Description        : Main program body.
*******************************************************************************/ 

/*
 *@Note
  USB设备的简易枚举过程例程：
  USBHDM(PB6)、USBHDP(PB7)。

*/

#include "debug.h"
#include "string.h"

/* Global define */


/* Global Variable */    
__align(4) UINT8  RxBuffer[ MAX_PACKET_SIZE ] ;      // IN, must even address
__align(4) UINT8  TxBuffer[ MAX_PACKET_SIZE ] ;      // OUT, must even address
__align(4) UINT8  Com_Buffer[ 128 ];                

/*******************************************************************************
* Function Name  : Set_USBConfig
* Description    : Set USB clock.
* Input          : None
* Return         : None
*******************************************************************************/
void USBHD_ClockCmd(UINT32 RCC_USBCLKSource,FunctionalState NewState)
{
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, NewState);
	EXTEN->EXTEN_CTR |= EXTEN_USBHD_IO_EN;
	RCC_USBCLKConfig(RCC_USBCLKSource);             //USBclk=PLLclk/1.5=48Mhz			 
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_USBHD,NewState);  
}
/*发送数据长度*/
UINT8	TXDUsbCH341BulkOut( PUINT8 pBuf,UINT8 len )  // 341批量发送
{
  UINT8 TxCnt,s,endp;
  endp = ThisUsbDev.GpVar[1];
	while ( len ) {
		Delay_Us( 200 );
		R8_UH_TX_LEN = len >= 32 ? 32 : len;
		for ( TxCnt = 0; TxCnt != R8_UH_TX_LEN; TxCnt ++ ) {
			TxBuffer[ TxCnt ] = *pBuf;
			pBuf ++;
		}
		s = USBHostTransact( USB_PID_OUT << 4 | endp & 0x7F, endp & 0x80 ? RB_UH_R_TOG | RB_UH_T_TOG : 0, 0 );  // OUT数据
		if ( s == ERR_SUCCESS ){ 
		len -= R8_UH_TX_LEN;
		endp ^= 0x80;  // 同步标志翻转
		ThisUsbDev.GpVar[1] = endp;  // 保存同步标志位
		}
    else return( s );
	}
  return ERR_SUCCESS;
}
/*返回数据接收长度，数据存放RxBuffer*/
UINT8	RXDUsbCH341BulkIn( void )  // 341批量接收
{
    UINT8 i,s,len,endp = ThisUsbDev.GpVar[0];
		s = USBHostTransact( USB_PID_IN << 4 | endp & 0x7F, endp & 0x80 ? RB_UH_R_TOG | RB_UH_T_TOG : 0, 0 );  // CH559传输事务,获取数据,NAK不重试
		if ( s == ERR_SUCCESS ) {
			endp ^= 0x80;  // 同步标志翻转
			ThisUsbDev.GpVar[0] = endp;  // 保存同步标志位
			len = R8_USB_RX_LEN;  // 接收到的数据长度
			if ( len ) {
//				printf("CH341 In data: \n");
//				for ( i = 0; i < len; i ++ ) printf("x%02X ",(UINT16)(RxBuffer[i]) );
//				printf("\n");
						  TXDUsbCH341BulkOut((RxBuffer),len );
			}
      return len;
		}
		else if ( s != ( USB_PID_NAK | ERR_USB_TRANSFER ) ) printf("CH341 error %02x\n",(UINT16)s);  // 可能是断开了
    return 0;
}



/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Return         : None
*******************************************************************************/
int main()
{	
	UINT8	s;
      UINT8 tmpBuf[4] = {'a','d','t','X'};
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
  Delay_Init();
	USART_Printf_Init(115200);
	printf("SystemClk:%d\r\n",SystemCoreClock);
	
	printf("USBHD   HOST Test\r\n");
	USBHD_ClockCmd(RCC_USBCLKSource_PLLCLK_1Div5,ENABLE); 
  pHOST_RX_RAM_Addr = RxBuffer;
  pHOST_TX_RAM_Addr = TxBuffer;
	USB_HostInit();
	printf( "Wait Device In\n" );
	
	while(1)
	{
		s = ERR_SUCCESS;
		
		if ( R8_USB_INT_FG & RB_UIF_DETECT )
		{  
			R8_USB_INT_FG = RB_UIF_DETECT ; 

			s = AnalyzeRootHub( );   
			if ( s == ERR_USB_CONNECT ) 
			{
				printf( "New Device In\r\n" );		
				FoundNewDev = 1;
			}
			if( s == ERR_USB_DISCON )
			{
				printf( "Device Out\r\n" );					
			}
		}
		
		if ( FoundNewDev || s == ERR_USB_CONNECT ) 
		{ 
			FoundNewDev = 0;
			Delay_Ms( 200 ); 
			s = InitRootDevice( Com_Buffer );  
			if ( s == ERR_SUCCESS ) printf( "Device Enum Succeed\r\n" );
			else printf( "Device Enum Failed\r\n" );
		}
		if(ThisUsbDev.DeviceStatus == ROOT_DEV_SUCCESS)
		{
			//printf("init ok\n");
//		  TXDUsbCH341BulkOut( tmpBuf,0 );
			RXDUsbCH341BulkIn(  );  // 341批量接收
		}
	}
}




